KB_REFERENCE_TEMPLATE = \
                """使用代码块中的内容作为你的辅助知识，并且根据这些知识按下面的要求回答问题:
                    ```{reference}```

                    回答要求：
                    - 仔细理解辅助知识和问题直接的关联，如果辅助知识中的内容和问题毫不相关，则拒绝回答问题。
                    - 如果答案中有引用辅助知识，请确保引用内容和辅助知识中一致，不要胡编乱造。
                    - 使用 Markdown 语法优化回答格式，按照逻辑分段回答。
                    - 使用与问题相同的语言回答。

                    问题是:{question}
                """